#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  fridgebot.py
#  
# Lloyd Matthews
# 

# Uses code from various sources to read data from a Raspberry Pi
# serial port and post it to an Adafruit IO feed.
### Not using subscribe feature. TBD use IFTTT to notify

# Adafruit IO code adapted from 
# https://github.com/adafruit/io-client-python/blob/master/examples/mqtt_client.py
# Example of using the MQTT client class to subscribe to and publish
#  feed values.
# Original Author: Tony DiCola

# need to import whole lib to prevent time.time() RTE
import time
# Import pySerial package for serial comm.
# package must be installed with 'pip3 install pyserial'.
#  docs: https://pyserial.readthedocs.io/en/latest
from serial import Serial, SerialException
# Import Adafruit_IO MQTT client package.
# package must be installed with 'pip3 install Adafruit_IO'.
#  docs: https://github.com/adafruit/io-client-python
from Adafruit_IO import MQTTClient, AdafruitIOError


# Timestamp format (YYYYMMDDHHMMSS)
STAMPFMT = '%Y%m%d%H%M%S'

# Serial port locations and baudrate
PORTS = ('ttyUSB0', 'ttyUSB1')
BAUDRATE = 9600

# Set client config file path.
# File has Adafruit IO username on first line, and
# AIO key on second line. Both are in PLAINTEXT.
AIO_CONFIG_PATH = './mqttAdafruitIO.cfg'

# Set feed ID and publishing interval.                       
AIO_FEED     = 'fridgebot'
PUBLISH_INTERVAL_SEC = 10


# Callback function: called when the client disconnects.
def disconnected(client):
    print('Disconnected from Adafruit IO!')
    quit(0)


def getConfig():
	# Get PLAINTEXT Adafruit IO username and key.
	# Note this is insecure since MQTT protocol is not encrypted.
	# TBD: use TLS and SSL certificate.
	# 'with' will close file when statement is done
	with open(AIO_CONFIG_PATH, 'r') as f:
		try: 
			username = f.readline().rstrip('\n')
			key      = f.readline().rstrip('\n')
		except IOError as e:
			print("getConfig(): " + str(e))
			quit(1)
	return(username, key)
	
	
# Get an open USB serial port, or quit if neither are open.
def openSerialPort():
	ser = None
	# Try both standard locations of Raspberry Pi 3 USB serial ports.
	# Specify timeout so readline() won't block forever if no
	#  newline received.
	for port in PORTS:
		try:
			ser = Serial('/dev/' + port, BAUDRATE, timeout=1)
		except SerialException as e:
			continue
	if ser is None:
		print("openSerialPort(): Neither USB serial port was open!")
		quit(1)
		
	ser.flushInput()
	print("USB serial port " + ser.port + " open.")
	return ser


# Now the program needs to use a client loop function to ensure messages
#  are sent and received.
# Pump the message loop yourself by periodically calling the client loop 
#  function.  Notice how the loop below changes to call loop 
# continuously while still sending a new message every 10 seconds.
# This is a good option if you don't want to or can't have a thread
#  pumping the message loop in the background.
def messageLoop(client, ser):
	print('Starting message loop for feed \'' + AIO_FEED +\
	'\' (press Ctrl-C to quit)...')

	time.sleep(5)
	last = 0
	while True:
		# Continuously pump the message loop.
		try:
			client.loop()
		# If client params are invalid, loop() throws RuntimeError,
		# not AdafruitIOError
		except Exception as e:
			print('loop(): ' + str(e))
			quit(1)
		
		# Send a new message only on interval.	
		if (time.time() - last) >= PUBLISH_INTERVAL_SEC:
			try:		
				# Read serial bytes and convert to string.
				response = ser.readline().decode()
			except Exception as e:
				print('Error reading from serial port: ' + str(e))
				ser.close()
				quit(1)
			else:
				# Create timestamp with current time.
				stamp = time.strftime(STAMPFMT);
				# Create CSV string of stamp and data.
				logStr = stamp + ',' + response
				# publish() creates feed if doesn't exist,
				# or appends to an existing feed.
				client.publish(AIO_FEED, logStr)
				print('Published: ' + logStr)
				last = time.time()	
		

def main():
	
	# Get PLAINTEXT client configuration.
	(aio_username, aio_key) = getConfig()

	# Create an MQTT client instance with PLAINTEXT username and key.
	client = MQTTClient(aio_username, aio_key)

	# Setup client callback on disconnect.
	client.on_disconnect = disconnected

	# Connect to the Adafruit IO server.
	client.connect()
	
	# Get an open serial port.
	ser = openSerialPort()
	
	# Message loop.
	messageLoop(client, ser)
	
	
if __name__ == '__main__':
	main()
